%%This is a part of multi-layered random forest (MLRF) framework

% find optimal number of clusters given the link, and the options
% the main options are minimal cluster size (options.small) and 
% required number of clusters of this size (options.clnum)

function [c,sizes,maxclcount] = optHieClustering(D,options)
    if (exist('options','var'))
        if (isfield(options,'small'))
            small = options.small; %this is not a class!
        else
            small = 5;
        end
    else
        small = 5;
    end
    
    clNumW = options.clnum;
    
    clSizes = ones(2*size(D,1)+1,1);
    clSizes((size(D,1)+1+1):end) = 0; %these are not known yet
    
    optLevel = -1;
    maxLevel = -1;
    
    maxclcount = -1;
    
    c = 1:(size(D,1)+1);
    
    %define if we already aggregated enough clusters
    agg = 0; 

    %now compute cluster sizes for each level
    for i=1:size(D,1)
        
        old1 = clSizes(D(i,1));
        old2 = clSizes(D(i,2));
        
        %merge sizes
        clSizes(size(D,1)+1+i) = clSizes(D(i,1)) + clSizes(D(i,2));

        %these do not exist separatelly any more
        clSizes(D(i,1)) = 0; 
        clSizes(D(i,2)) = 0;
        
        clcount = sum(clSizes>small);

        
        if(clcount>=clNumW)
            agg = 1;
        end
        
        if(clcount >maxclcount)
            maxclcount = clcount;
            maxLevel = i;
        end
        
        if (agg && sum(clSizes>small)<clNumW)
            %we merged too many clusters
            optLevel = i-1;
            clSizes(D(i,1)) = old1;
            clSizes(D(i,2)) = old2;
            %compute clustering
            break;
        end 
        
        %merge classes
        c(c==D(i,1)) = size(D,1)+1+i;
        c(c==D(i,2)) = size(D,1)+1+i;     
        
        optLevel = optLevel+1;
        
    end
    
    %relabel classes
    unic = unique(c);
    
    sizes = zeros(size(unic));
    
    for i=1:length(unic)
        c(c==unic(i)) = i;
        sizes(i) = clSizes(unic(i));
    end
    
end